package org.openxma.dsl.generator;

import java.util.Properties;

import org.eclipse.emf.common.util.URI;

public interface GeneratorConfiguration {
    /**
     * @return the fileExtension of the model files which are relevant for this configuration
     */
    String getFileExtension();

    /**
     * @return the configured <code>NamingStrategy</code> strategy
     * @see NamingStrategy
     */
    NamingStrategy getNamingStrategy();

    /**
     * @return the configured <code>ModelModifier</code> strategy
     * @see ModelModifier
     */
    ModelModifier getModelModifier();

    /**
     * @return the configured <code>LayoutStrategy</code> strategy
     * @see LayoutStrategy
     */
    LayoutStrategy getLayoutStrategy();

    /**
     * Searches for the property with the specified key in this property list.
     * 
     * @param key
     *            the property key.
     * @return the value in this property list with the specified key value.
     */
    String getProperty(String key);

    /**
     * Searches for the property with the specified key in this property list.
     * The method returns the default value argument if the property is not
     * found.
     * 
     * @param key
     *            the property key.
     * @param defaultValue
     *            a default value.
     * 
     * @return the value in this property list with the specified key value.
     */
    String getProperty(String key, String defaultValue);

    /**
     * @return the configured URI for the project containing the models to generate
     */
    URI getProjectFolder();
    /**
     * @return the configured URI for source files (e.g. *.java)
     */
    URI getSourceFolder();

    /**
     * @return the configured URI for resource files (e.g. *.properties,*.xml)
     */
    URI getResourceFolder();

    /**
     * @return the configured URI for test source files (e.g. *Test.java)
     */
    URI getTestSourceFolder();

    /**
     * @return the configured URI for generated source files (e.g. *Gen.java)
     */
    URI getGenSourceFolder();

    /**
     * @return the configured URI for generated resource files (e.g. *.properties,*.xml)
     */
    URI getGenResourceFolder();

    /**
     * @return the configured URI for generated test source files (e.g.  *TestGen.java)
     */
    URI getGenTestSourceFolder();

    static class INSTANCE {
        private static final ThreadLocal<GeneratorConfiguration> configurationHolder = new ThreadLocal<GeneratorConfiguration>();

        /**
         * Reset the GeneratorConfiguration for the current thread.
         */
        public static void reset() {
            configurationHolder.set(null);
        }

        /**
         * Associate the given GeneratorConfiguration with the current thread,
         * <i>not</i> exposing it as inheritable for child threads.
         * 
         * @param generatorConfiguration
         *            the current GeneratorConfiguration, or <code>null</code>
         *            to reset the thread-bound context
         */
        public static void set(GeneratorConfiguration generatorConfiguration) {
            configurationHolder.set(generatorConfiguration);
        }

        /**
         * Return the GeneratorConfiguration associated with the current thread,
         * if any.
         * 
         * @return the current GeneratorConfiguration
         */
        public static GeneratorConfiguration get() {
            GeneratorConfiguration generatorConfiguration = configurationHolder.get();
            if (generatorConfiguration == null) {
                generatorConfiguration = setProperties(null);
            }
            return generatorConfiguration;
        }

        /**
         * Associate the given generator properties with the current thread.
         * <p>
         * Will implicitly create a GeneratorConfiguration for the given
         * properties.
         * 
         * @param properties
         *            the dsl <code>Properties</code> used by the generator, or
         *            <code>null</code> to reset the thread-bound context
         */
        public static GeneratorConfiguration setProperties(Properties properties) {
            GeneratorConfiguration generatorConfiguration = new DefaultGeneratorConfiguration(properties);
            set(generatorConfiguration);
            return generatorConfiguration;
        }

        public static ModelModifier getModelModifier() {
            return get().getModelModifier();
        }

        public static LayoutStrategy getLayoutStrategy() {
            return get().getLayoutStrategy();
        }

        public static NamingStrategy getNamingStrategy() {
            return get().getNamingStrategy();
        }

        public static String getProperty(String key) {
            return  get().getProperty(key);
        }
        
        public static URI getSourceFolder() {
            return get().getSourceFolder();
        }

        public static URI getResourceFolder() {
            return get().getResourceFolder();
        }

        public static URI getTestSourceFolder() {
            return get().getTestSourceFolder();
        }

        public static URI getGenSourceFolder() {
            return get().getGenSourceFolder();
        }

        public static URI getGenResourceFolder() {
            return get().getGenResourceFolder();
        }

        public static URI getGenTestSourceFolder() {
            return get().getGenTestSourceFolder();
        }

        public static String getDaoBaseClassname() {
            return getProperty(GeneratorConstants.DOMAIN_MODEL_DAO_BASE_CLASS);
        }

        public static String getServiceBaseClassname() {
            return getProperty(GeneratorConstants.DOMAIN_MODEL_SERVICE_BASE_CLASS);
        }

        public static String getHibernateDialect() {
            return getProperty(GeneratorConstants.HIBERNATE_DIALECT_CLASS);
        }

    }

}
